;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x48dad3a0e7a1ffd174bbb01a5e818ad8 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x42b9641a01e111605e9e6cc987d448be . "./code.cg2js/cg-printing.fasl")
(#x3ded5bc909301c74f2a061622a650e26 . "./code.cg2js/cg-caret.fasl")
(#x23ef543c2dcb1cfc0c53c11f83964aff . "./code.cg2js/cg-item-list.fasl")
(#xd68b7d312cb67054901e1b7adb60ab16 . "./code.cg2js/cg-progress-indicator.fasl")
(#x7113552b02a682c972a840332ab5354d . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x293e580e22603b20900ebbe7033d515f . "./code.cg2js/cg-outline.fasl")
(#xca1cef03d4b76ca903f5e2e7f25153c3 . "./code.cg2js/cg-palette.fasl")
(#xbed31641803bcfe023fc517b932bca59 . "./code.cg2js/cg-selected-object.fasl")
(#xacb89a440e56fcd1ca8c129d701ab384 . "./code.cg2js/cg-status-bar.fasl")
(#xe55c076362a5e861113d44bb5a863bb5 . "./code.cg2js/cg-wrap-string.fasl")
(#x00891a4171d347e17ab3a6a66dd08f4b . "./code.cg2js/cg-fill-texture.fasl")
(#x93926634079411d9cec5ab75f0a6c87d . "./code.cg2js/cg-layout.fasl")
(#x13a1d66f17909f28da7a5e2faffb486c . "./code.cg2js/cg-timer.fasl")
(#xb0f7c6822a21d06d2bf6e766623b3283 . "./code.cg2js/cg-dialog-item.fasl")
(#x5b662e92099d516af16fe2df64719be2 . "./code.cg2js/cg-button.fasl")
(#x6141578bf4915a7cb59e0e89e344e65c . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x993172493336aa731f647447e35b0ce2 . "./code.cg2js/cg-tray.fasl")
(#xe0f40f1f64d957375b86e839914e1ec5 . "./code.cg2js/cg-plot-widget.fasl")
(#x37fb7aa7134e5193261eaba7ebc6b757 . "./code.cg2js/cg-pixmap.fasl")
(#xd51c239852b9f6cebaacdbf179c3cec0 . "./code.cg2js/cg-font-dialog.fasl")
(#xc53a3cd56ce4fddea5db76157bcb71ee . "./code.cg2js/cg-os-widget.fasl")
(#x5e618d108ff580154b7aff35eeb86dd8 . "./code.cg2js/cg-text-or-combo.fasl")
(#xb287a89e84fa57a6dfc42a4eafdfc619 . "./code.cg2js/cg-trackbar.fasl")
(#xba24d994ec6c2a697514c503d741e016 . "./code.cg2js/cg-up-down-control.fasl")
(#xba9e53c55d3ab922f58e4dd866531ac2 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xc0ba1e6dc622b2a232214dafc849ec3f . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x76893d0d332a49901b9c255147cb2234 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x50b2e9a090cdd2feb3c584391123a61a . "./code.cg2js/cg-bitmap-stream.fasl")
(#xb6396474000807af78150dfc9985e304 . "./code.cg2js/cg-chart-widget.fasl")
(#xeed0038c9012d3b6f9b0002b6a755807 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x017a4b30e008d0e37bd664f1e9acaa9e . "./code.cg2js/cg-group-box.fasl")
(#xb761a8f193e16793f40ae611499f1e72 . "./code.cg2js/cg-static-text.fasl")
(#x7ad1f73a262d7cac667f96b7a2e2663b . "./code.cg2js/cg-class-slot-grid.fasl")
(#x92510b706946129a524baf2c61fd0883 . "./code.cg2js/cg-color-dialog.fasl")
(#xbdf20f43a317b498a4b33825ab59d455 . "./code.cg2js/cg-intersect.fasl")
(#x5624b934fa6b381f9beb294696feafe6 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xaafee5501432352c338732f7c99f6238 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x514bb6af14da588a0c3564e543a39381 . "./code.cg2js/cg-html-dialog.fasl")
(#xae10ecbd697b950c40055ab5b66538c4 . "./code.cg2js/cg-lettered-menu.fasl")
(#x3c19b5268d418e73b2acf1da946c1bc7 . "./code.cg2js/cg-class-grid.fasl")
(#x7c29387ab51d4434df929a96de690ecb . "./code.cg2js/cg-pixmap-printing.fasl")
(#x8e79ad3adc1ef3053cae021229550680 . "./code.cg2js/cg-calendar.fasl")
(#xc90976f29bd3c0988e2e7ca3cff01945 . "./code.cg2js/cg-editable-text.fasl")
(#x3eb03b4a9a33090613d6f51d5769598f . "./code.cg2js/cg-property.fasl")
(#x5e10f02877fd2b039e21203b892e3c45 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x807a7f53ca3939fa6182342c0f98bbbe . "./code.cg2js/cg-grid-widget.fasl")
(#x7e346d47c6d34a6d78c5cbaa526e484c . "./code.cg2js/cg-paren-matching.fasl")
(#x1046301dbfb3def7e2b33d3557cb80a7 . "./code.cg2js/cg-dev.fasl")
(#xab9eb1a6b2aae4006cb2bb5dd2f7538b . "./code.cg2js/cg-drawable.fasl")
(#xf5bf100c06deb07c678d6cac23569d4f . "./code.cg2js/cg-curve.fasl")
(#x1775cd50098bae69e7b63d9eeeba44c7 . "./code.cg2js/cg-lisp-widget.fasl")
(#x5be8cfc741ed5549d33d2280f6a3b390 . "./code.cg2js/cg-dde-utils.fasl")
(#xbc4694f3350bfdab0e64ba12fc8a6919 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x5b469f8dbe19decb028907fd5284faf5 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xf683389e4f5347a6f0cc22f06aca74ee . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x6abf2ade2f96922d1defa62d66b273d3 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xaf62067c96c966278d9df2b15bbcbb6b . "./code.cg2js/cg-lamp.fasl")
(#x39d68ab9e07ffc2ba0d04793f3cf58f2 . "./code.cg2js/cg-common-control.fasl")
(#xe0bb0f313caf7821e37c54cd75b0432b . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x82683a4f4740e91a3c992b63b5f376e4 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x78761f3819f7bf1ae960bb672912fdff . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x5019dd09884646ea0c20d0d4ba644559 . "./code.cg2js/cg-project-window.fasl")
(#x460e265fc132dc47ad926fa36d0d7daf . "./code.cg2js/cg-sample-file-menu.fasl")
(#x232cd2c3d3d227236a30f1868d3e64d0 . "./code.cg2js/cg-class-support.fasl")
(#xf49c845ecf1aa627d1e42a1e2c84d957 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x86a6f6797e8591d3184db1f1bf20df48 . "./code.cg2js/cg-yes-no-list.fasl")
(#x5aa6f199d34d7b20a71770977e39babd . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x9dfe92f2b50fee98ae05c0656a0ced80 . "./code.cg2js/cg-string-dialog.fasl")
(#xad524aad0d68627aa47da2fa18698224 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xd372b571ff75c4024ea8e591f7174f1e . "./code.cg2js/cg-dropping-outline.fasl")
(#x720b24e7d5b65b07b9401d49a59d35d4 . "./code.cg2js/cg.fasl")
(#x91117ba26a5d5d7de9a895e184243a41 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x5b84b74dafde0583c7d0892d214aeba2 . "./code.cg2js/cg-scaling-stream.fasl")
(#x606f31d835573024ed788edda9bd71b9 . "./code.cg2js/cg-yes-no-string.fasl")
(#xe8d2f0e52fcdb86b4d2ff5595b7be781 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x65b15429ac3a675c1881e2f09a20f492 . "./code.cg2js/cg-web-browser.fasl")
(#x56d13f1a66880540d556de8860094f91 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xff7617d3915d282bf119bb1cb4a15f12 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x9c124ec6c85036992b7aa7c2bde541e1 . "./code.cg2js/cg-message-dialog.fasl")
(#x627506b081c4bf5fa0e35a11d7e53da7 . "./code.cg2js/cg-comtab.fasl")
(#xd4d1e1e93eb8e1fd0668bb82ad7b75dc . "./code.cg2js/cg-base.fasl")
(#x9230399f5183e5ec50da529acfbca9cb . "./code.cg2js/cg-pixmap-widget.fasl")
(#x2d15227184441a7a17dcf972c53e8ab0 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xe4cda47fc0b3886d7abc57284f990716 . "./code.cg2js/cg-utility-dialog.fasl")
(#xbf77d11bba76d90570a07b06c64fcf05 . "./code.cg2js/cg-object-editor.fasl")
(#xfd5cb10a1643290931395d450952c5c5 . "./code.cg2js/cg-edit-in-place.fasl")
(#x20034959d226e361650ddcff9c34162d . "./code.cg2js/cg-radio-button.fasl")
(#xbd5abd2facdb52fb749699fda45629b2 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x86a3cdffb21d6bbed1ba76eed03b3994 . "./code.cg2js/cg-nodes.fasl")
(#xf7691499dca2980cc20c55b2f5d323d6 . "./code.cg2js/cg-get-position.fasl")
(#xe204f966cb3c60654013689ec8c9dec6 . "./code.cg2js/cg-directory-dialog.fasl")
(#x78e5d69d0bebaf778a48a23ee8c00335 . "./code.cg2js/cg-lisp-text.fasl")
(#xfc684ee52a1d746fdb7a381bb1fb7b76 . "./code.cg2js/cg-combo-box.fasl")
(#xbabe10d9e1e667cc0f8d736b28c86291 . "./code.cg2js/cg-text-widget.fasl")
(#x5b3be604065fb5bebaaaef0e22461ec9 . "./code.cg2js/cg-list-view.fasl")
(#x8828ed82d1a1b4ce272d3b40fe58b98a . "./code.cg2js/cg-graphics-context.fasl")
(#x05780ea7fc91e2523106b8df6a58aa98 . "./code.cg2js/cg-get-pixmap.fasl")
(#xde6429fc5aeae3e3f3b9315cd5604708 . "./code.cg2js/cg-menu.fasl")
(#x97071ccdd751c25fa0fdb65d5108246a . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x9473d1dcabbafcd5e09e5a99854c85bc . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x38e60283cee661d11916cbdf46c26294 . "./code.cg2js/cg-tooltip.fasl")
(#x65e0c275d7327568b8e768e8a5170726 . "./code.cg2js/cg-html-widget.fasl")
(#x3464a60883affc4dc5157e326c8e89a5 . "./code.cg2js/cg-choice-list.fasl")
(#x654e908b8435b641bef07624f82266dd . "./code.cg2js/cg-icon.fasl")
(#x205675dda59bd8920648f7a68883b7a8 . "./code.cg2js/cg-acache.fasl")
(#xc3657ac1bdbd7df616d93e617089bd42 . "./code.cg2js/cg-template-string.fasl")
(#x345db14a992bd7ccc71d6875aece624d . "./code.cg2js/cg-bitmap-pane.fasl")
(#xdc094569e88f9746a944f66ac0e1fa7a . "./code.cg2js/cg-toggling-widget.fasl")
(#xadd90972c75b9e9a2ffff9a3e1b55a50 . "./code.cg2js/cg-tab-control.fasl")
(#xa753bc1a525dc155e7429387aaf55a01 . "./code.cg2js/cg-split-bar.fasl")
(#x7c979ec493b3a0d710c807700bbe2836 . "./code.cg2js/cg-picture-widget.fasl")
(#xadb0b05e63a7d295e77bc2841d0f90d6 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x8c3dd01048001ad32127e73971573844 . "./code.cg2js/cg-choose-printer.fasl")
(#x4d78b3da015d9226af9a328865ffd777 . "./code.cg2js/cg-os-window.fasl")
(#x3f63a2877970bd241977333f8868abbd . "./code.cg2js/cg-scroll-bar.fasl")
(#x89a81c57243a22b53ab491176b6343f4 . "./code.cg2js/cg-clipboard.fasl")
(#xfe5b6a3dda87e236e66412afeaae753a . "./code.cg2js/cg-find-string-dialog.fasl")
(#x7376dfdf4a0c2f83714bc2ca27379d71 . "./code.cg2js/cg-hotspot.fasl")
(#x505efc343fe3cdb694e8bf237ae02188 . "./code.cg2js/cg-check-box.fasl")
(#x8b3aa2885727c664daadcb2e5b474467 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x14cff3a1a71e782e52b43e47334703c3 . "./code.cg2js/cg-file-dialog.fasl")
(#x69c16ae048a433d69f372ea9d98f01bc . "./code.cg2js/cg-toolbar.fasl")
(#x3af694317b6bc89e5983bab5932e36c6 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x79fae926d0deab741c93fda2c578cdf6 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x9afae4c75a56f8a66c085e8ccc0fbffd . "./code.cg2js/cg-shortcut-menu.fasl")
(#x482f574dc2425e7551b3364b58d56f1e . "./code.idejs/ide-project.fasl")
(#x0978143036ab6ef7a4734c3c9d2a5e74 . "./code.idejs/ide-intro.fasl")
(#x828ff3187ca9db496adcbc164e1718fe . "./code.idejs/ide-find-in-files.fasl")
(#xd531cfb3f93ec1e12e1d4495ee650072 . "./code.idejs/ide-grapher.fasl")
(#xd26b4c554380ef89f8ad8d297f14c7b0 . "./code.idejs/ide-inspector.fasl")
(#x148f1130b09c31ca33a63f7fd3f49f43 . "./code.idejs/ide-editor.fasl")
(#xbc100ecc04610357bb4f2f51e8b4883e . "./code.idejs/ide-window-list.fasl")
(#x175bde5d55301ee6f0ca9a3cfcf0881d . "./code.idejs/ide-process-browser.fasl")
(#x7c094ee3a4a2564c1dccc23867c946b2 . "./code.idejs/ide-patch-dialog.fasl")
(#xef9f4edcf1c855c99f2002cab264c1ce . "./code.idejs/ide-class-grapher.fasl")
(#x6619c46fdbe31c6abc2009a86f9dfc2e . "./code.idejs/ide-base.fasl")
(#xf2996eb9f3211b079ebdd87e543094d4 . "./code.idejs/ide.fasl")
(#xe924e28ee2201b2902ff83c3b95f18cc . "./code.idejs/ide-trace.fasl")
(#x16a6bdc153af8e0b1a495165a30e659d . "./code.idejs/ide-form.fasl")
(#x71fc410d28b328370e6f01030e52630f . "./code.idejs/ide-find-definitions.fasl")
(#x10387216dae4dcf7e164a2681f62c341 . "./code.idejs/ide-apropos.fasl")
(#x2ab960dd70f67641f06aa21520551430 . "./code.idejs/ide-class-browser.fasl")
(#x680f015c2ce259918f927992bc5dbe7d . "./code.idejs/ide-navigator.fasl")
(#x29d0e3a721c7ae5031eeec52ea99d77f . "./code.idejs/ide-clipboard.fasl")
(#x1b83a98c1838636bb7a7999b1520f41f . "./code.idejs/ide-acache.fasl")
(#x427104d30b38fab9eb7aeb2446fcd67b . "./code.idejs/ide-keyshort.fasl")
(#x9254ccc6de0b1cbefe751dadb8af6606 . "./code.idejs/ide-package-list.fasl")
(#xf969012c8024f288fd23bc68dfd4c402 . "./code.idejs/ide-dev.fasl")
(#x696b64509ebe0538a3b58ac1081d4fa0 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
